% Program for calculating average delay for slotted ALOHA. s is the arrival
% probability in a slot, a is the probaility of transmission in a slot
% and n is the total number of terminals. The output is: 
% 1) the average number of backlogged terminals from equation (5.67)
% 2) Average message delay from equation (5.69)

function y=ex57(s,a,n)
% The first step is to fill in the transition matrix t. initialization
t=zeros(n+1,n+1);
%equation (5.59)
t(1,1)=(1-s)^n + n*s*(1-s)^(n-1);
%equation (5.60)
for j=2:n
    t(1,j+1)=comb(n,j)*s^j*(1-s)^(n-j);
end
%equation (5.62)
for i=1:n
    t(i+1,i)=i*a*((1-a)^(i-1))*(1-s)^(n-i);
%As noted in the text, if this number is 0 for some value of i, the system
%is unstable. We check for this now.
    if t(i+1,i) < 10^-20
        flag=t(i+1,i)
    end
end
%equation (5.63)
for i=1:n
    t(i+1,i+1)=s*(n-i)*(1-s)^(n-i-1)*(1-a)^i+((1-a)^i)*((1-s)^(n-i))+(1-s)^(n-i)*(1-(1-a)^i-i*a*(1-a)^(i-1));
end
%equation (5.64)
for i=1:n-1
    t(i+1,i+2)=s*(n-i)*(1-s)^(n-i-1)*(1-(1-a)^i);
end
%equation (5.65)
for i=1:n-1
    for j=2:n-i
        t(i+1,i+j+1)=comb(n-i,j)*s^j*(1-s)^(n-i-j);
    end
end
%Check: All of the rows of the transition matrix should sum to one.
for i=1:n+1
    sum=0;
    for j=1:n+1
        sum=sum+t(i,j);
    end
    check=sum;
end  
% The next step is to find the buffer occupancy probabilities from equation
% (5.66)
p(1)=1;%intialization
for l=1:n
    sum=0;
    for k=1:l
        sum=sum+p(k)*t(k,l);
    end
    p(l+1)=(p(l)-sum)/t(l+1,l);
end
%The next step is to normalize the terms of p(.) similarly to equation
%(5.9).
norm=0;
for i=1:n+1
    norm=norm+p(i);
end
for i=1:n+1
    p(i)=p(i)/norm;
end
%now we find the mean number in the system (equation (5.67)).
kbar =0;
for i=1:n+1
    kbar=kbar+p(i)*(i-1);
end
%From equation (5.68), we find the mean arrival rate
sin=s*(n-kbar);
%Finally, average delay from Little's formula.
avedel=1+ kbar/sin;
y(1)=sin;
y(2)=avedel;
return
    
    

