%In this program we carry out the calculations for example 7.4, the survivor function for the infinite buffer case. 
%We build on the results from Examples 7.2 and 7.3. 

function w=ex74(Cp,N,a,b,R)
%Cp=channel capacity, N=number of binary sources, a=flow to active stat,
%b=flow to idle state and R=rate of single source in the active state.
%Recall that R cannot be such that C-iR=0 for i=1,2,...,N.

%Now check the load with respect to capacity.
load=N*R*a/(a+b)
if load >= Cp%Indicates overload and there is no point in going further. 
    stop
end

%steady-state probabilities
for i=0:N
    p(i+1)=comb(N,i)*(a/(a+b))^i*(b/(a+b))^(N-i);
end

for k=0:fix(N/2);% As mentionned in the text, the terms A, B and C are symmetric about N/2.
    
    A(k+1)=(R*(k-N/2))^2-(Cp-R*N/2)^2; %Start with equation (7.33).
    B(k+1)=2*R*(b-a)*(k-N/2)^2+N*(a+b)*(Cp-R*N/2);
    C(k+1)=((a+b)^2)*((k-N/2)^2-(N/2)^2);
    
end
    
 for kk =0:fix(N/2)
    e(kk+1)=(-B(kk+1)-sqrt(B(kk+1)^2-4*A(kk+1)*C(kk+1)))/(2*A(kk+1)); %First of pair of eigenvalues.
    e(N-kk+1)=(-B(kk+1)+sqrt(B(kk+1)^2-4*A(kk+1)*C(kk+1)))/(2*A(kk+1));%Second
end

%When N is even, the eignvalue for k=N/2 is computed two, but the coding is simplied. 
    
nopos = fix(Cp/R);%As a check we should have this many positive roots as well as a root at 0.
    

%Up to this point we have repeated the calculations in Example 7.2. Now we
%find the eigenvectors using equation (7.37).
for k=0:N %First, we compute the terms in equation 7.28).
    r1(k+1)=(-(e(k+1)*R+b-a)+sqrt((e(k+1)*R+b-a)^2+4*a*b))/(2*a);
    r2(k+1)=(-(e(k+1)*R+b-a)-sqrt((e(k+1)*R+b-a)^2+4*a*b))/(2*a);
end

%Now, we compute the exponent in equation (7.31) as given by (7.30). We
%expect these to be integers.

for k=0:N
    g(k+1)=(e(k+1)*Cp+N*a*r1(k+1)-N*a)/(a*(r1(k+1)-r2(k+1)));
end

%Now, the double sum in (7.7) for each value of i, which corresponds to the eigenvalues,
%to produce the compnents of the eigenvalues.
for i=0:N %N+1 eigenvalues
    gg=fix(g(i+1));%cleans up roundoff error.
    for j=0:N
    sum=0;
        for m=0:gg
            sum=sum + comb(gg,m)*comb(N-gg,j-m)*(r1(i+1)^(g(i+1)-m))*(r2(i+1)^(N-j-g(i+1)+m));
        end
    phi(j+1,i+1)=(-1)^(N-j)*sum;
    end
end
%As a check, we compute the first two eigenvectors by equations (7.38) and (7.40).
for n=0:N
    phi0(n+1)=comb(N,n)*(b/a)^(N-n);
    phi1(n+1)=comb(N,n)*(N*R/Cp-1)^(N-n);
end

% Now, we find the coefficients.

upper=N-fix(Cp/R)-1;
factor = (a/(a+b))^N;
for j= 0:upper
    term=-factor;
    for i=0:upper
        if i ~= j
            coef(j+1)=term*e(i+1)/(e(i+1)-e(j+1));
        else 
            coef(j+1)=term;
        end
        term=coef(j+1);
    end
end
 
%Next, the terms of the survivor function.
for i=0:upper
    surv(i+1)=0;
    for j=0:N
        surv(i+1)=surv(i+1)+phi(j+1,i+1);
    end
    surv(i+1)=surv(i+1)*coef(i+1);
end
x=real(surv);%Cleans up surrious imaginary components due to roundoff error.

for x=0:1000
    increm(x+1)=x/(100*abs(e(1)));
    G(x+1)=0;
    for i=0:upper
        G(x+1)=G(x+1)-surv(i+1)*exp(e(i+1)*increm(x+1))
    end
end
ww=real(G);
w=[increm;ww];




        