%In this program we consider the Token buffer case of example 7.6. We
%generate an array which gives the probability of buffer overflow in the
%case of finite buffer. 
% Note use Token rate for the value Cp, and Bffr for Bt+Bd. Read contents 
% in Example 7.6.
function w=ex76(Cp,N,a,b,R,Bffr)
%Cp=Token Rate, N=number of binary sources, a=flow to active stat,
%b=flow to idle state and R=rate of single source in the active state.
%Recall that R cannot be such that C-iR=0 for i=1,2,...,N. Finally, Bffr is
%the sum of buffer sizes of Bt and Bd.

%Now check the load with respect to capacity.
load=N*R*a/(a+b)
if load >= Cp%Indicates overload and there is no point in going further. 
    return
end

for k=0:fix(N/2);% As mentionned in the text, the terms A, B and C are symmetric about N/2.
    
    A(k+1)=(R*(k-N/2))^2-(Cp-R*N/2)^2; %Start with equation (7.33).
    B(k+1)=2*R*(b-a)*(k-N/2)^2+N*(a+b)*(Cp-R*N/2);
    C(k+1)=((a+b)^2)*((k-N/2)^2-(N/2)^2);
   
end
    
 for kk =0:fix(N/2)
    e(kk+1)=(-B(kk+1)-sqrt(B(kk+1)^2-4*A(kk+1)*C(kk+1)))/(2*A(kk+1)); %First of pair of eigenvalues.
    e(N-kk+1)=(-B(kk+1)+sqrt(B(kk+1)^2-4*A(kk+1)*C(kk+1)))/(2*A(kk+1));%Second
end
%When N is even, the eignvalue for k=N/2 is computed two, but the coding is simplied. 
    
nopos = fix(Cp/R);%As a check we should have this many positive roots as well as a root at 0.
    

%Up to this point we have repeated the calculations in Example 7.2. Now we
%find the eigenvectors using equation (7.37).
for k=0:N %First, we compute the terms in equation 7.28).
    r1(k+1)=(-(e(k+1)*R+b-a)+sqrt((e(k+1)*R+b-a)^2+4*a*b))/(2*a);
    r2(k+1)=(-(e(k+1)*R+b-a)-sqrt((e(k+1)*R+b-a)^2+4*a*b))/(2*a);
end

%Now, we compute the exponent in equation (7.31) as given by (7.30). We
%expect these to be integers.

for k=0:N
    g(k+1)=(e(k+1)*Cp+N*a*r1(k+1)-N*a)/(a*(r1(k+1)-r2(k+1)));
end

%Now, the double sum in (7.7) for each value of i, which corresponds to the eigenvalues,
%to produce the compnents of the eigenvalues.
for i=0:N %N+1 eigenvalues
    gg=fix(g(i+1));%cleans up roundoff error.
    for j=0:N
    sum=0;
        for m=0:gg
            sum=sum + comb(gg,m)*comb(N-gg,j-m)*(r1(i+1)^(g(i+1)-m))*(r2(i+1)^(N-j-g(i+1)+m));
        end
    phi(j+1,i+1)=(-1)^(N-j)*sum;
    end
end

%Calculate the LHS of equation (7.52)

top=fix(Cp/R);
for i=0:N
    if i <= top
        p(i+1)=comb(N,i)*(a/(a+b))^i*(b/(a+b))^(N-i);
    else
        p(i+1)=0;
    end
end
lhs=p.';
%Now, calculate the matrix on the RHS.

for i=0:top
    for j=0:top
        mrhs(i+1,j+1)=phi(i+1,j+1)*exp(e(j+1)*Bffr);
    end
end
for i=0:top
    for j=top+1:N
        mrhs(i+1,j+1)=phi(i+1,j+1);
    end
end
for i=top+1:N
    for j=0:top
        mrhs(i+1,j+1)=phi(i+1,j+1);
    end
end
for i=top+1:N
    for j=top+1:N
        mrhs(i+1,j+1)=phi(i+1,j+1)*exp(-e(j+1)*Bffr);
    end
end

mrhs=real(mrhs);%There are spurious imaginary components due to roundoff error. 
%Is is now a simple step to invert the matrix and multoply in order to find
%the coefficients.
coef=inv(mrhs)*lhs;
%The coefficients need to be adjusted by the exponential of the buffer
%size.
for i=top+1:N
    coef(i+1)=coef(i+1)*exp(-e(i+1)*Bffr);
end

x=coef

for i=0:N
    surv(i+1)=0;
    for j=0:N
        surv(i+1)=surv(i+1)+phi(j+1,i+1);
    end
    surv(i+1)=surv(i+1)*coef(i+1);
end
x=real(surv);%Cleans up surrious imaginary components due to roundoff error.

for x=0:(50*Bffr)
    increm(x+1)=x/(100*abs(e(1)));
    G(x+1)=0;
    for i=0:N
        G(x+1)=G(x+1)+surv(i+1)*exp(e(i+1)*increm(x+1));
    end
end
ww=real(G);
w=[increm;ww];
