%In this example, we are evaluting Equations (8.74) and queue length distribution at departure points(8.49) 
%The inputs are: 
% R a (LxL) matrix, element r(i,j)= tranition rate from i to j 
% LMBD a a L dimansional vector giving the rates for each phase.
% m the message length of  messages, when the system is busy
% ms is the message length of first arriving message to the empty system

function [P0,P1,P2] = ex87(R,LMBD,m,ms)

%We get D0 and D1 from example 8.1.
fetch = ex81(R,LMBD);
D0=fetch(:,:,1);
D1=fetch(:,:,2);
D = D0 + D1; 
 I = [ 1 0; 0 1];
L=size(R,1);%The number of phases


[V,EG]=eig(D');%V is the matrix, whose columns are the  eigenvectors of D'. The diagonal elements of EV are the eigenvalues.

%We find the number of the eigenvalue which is equal to zero. 
index=1;
if EG(index,index) == 0
    x=abs(V(:,1));%The solution is the eigenvector corresponding to the zero eigenvalue.
else x=abs(V(:,1));index=index+1;
end

%The last step in solving (8.3) is normalizing so the the sum of the elements is one.
pi=x'/sum(x);%These are the phase probabilities.

%Finally, we do the calculation in (8.10) in order to check the load.

avelmbd=pi*D1*ones(L,1);
rho=avelmbd*m%This should be less than one; otherwise the system is overloaded.


%Now, we proceed to the iteration in (8.58). 
G0=zeros(L);
G1=eye(L);
count=0;
while (abs(det(G0-G1))>10^-30)
    G0=G1;
    G1=expm((D0+D1*G0)*m);
    e=ones(L,1);%Unitary vector
    test =G1*e;
    G1(1,:)=G1(1,:)/test(1);%normalizing the matrix to be stochastic
    G1(2,:)=G1(2,:)/test(2);
    count=count+1;%Keeping track of the number of iterations
end
G=G1;%Change of variable name to conform to the text.
%At this point we have the solution to (8.59).
term=count;%This is the count of the number of iterations to solve (8.59)

%The next step is to calculate (8.68)
K=-inv(D0)*D1*G;%This should be stochasitic matrix.

%In order to continue, we next need to solve (8.60), for g. The first step
%is to find the eigen values and the eigenvectors.
[V,EG]=eig(G');

%We need to find the number of the eigenvalue equal to one.
ind=1;
if round(EG(ind,ind))==1
    y=V(:,ind);
else
    ind=ind+1;
end
g=y'/sum(y);%The solution to (8.60) should be a row vector. It should also be normalized.


%The next step is to calculate alpha and  nu in (8.71).
e=ones(L,1);%Column vector of ones.
A=expm(D*m);%Equation (8.37).
alpha=e*avelmbd*m+inv(e*pi+D)*(A-eye(L))*D1*e;
nu=-inv(D0)*D1*(eye(L)-G+e*g)*inv((eye(L)-A+(e-alpha)*g))*e;

%Next we solve the eigenvalue equation in (8.72).
[V,EG]=eig(K');%The first column of VV is the eigenvector corresponding to the eigenvalue 1 for 
%the matrix K.
%We need to find the number of the eigenvalue equal to one.
ind=1;
if round(EG(ind,ind))==1
    z=V(:,ind);
else
    ind=ind+1;
end
k=z'/sum(z);%The solution to (8.60) should be a row vector. It should also be normalized.

%The last step is the calculation of the zero-level probability in (8.73).

P0=k/(k*nu)


final=P0;

Mi = 276;   % use the value of 300 for the case of mu1 = 10, mu2 = 40 and mu3 = 40
A1 = zeros(2,(Mi+1)*2);
B1 = zeros(2,(Mi+1)*2);
%Now we calculate the matrices A(..)
A(:,1:2) = expm(D0*m);
A1(:,1:2) = expm(D0*ms);
B1(:,1:2) = (-1)*D0^(-1)*D1*A1(:,1:2);
factl = 1;
for n = 1:Mi,
  factl = factl * n;
 A(:,n*2+1:(n+1)*2)  = expm(D0*m)*(D1*m)^n/factl;
 A1(:,n*2+1:(n+1)*2)  = expm(D0*ms)*(D1*ms)^n/factl;
 B1(:,n*2+1:(n+1)*2)  = (-1)*D0^(-1)*D1*A1(:,n*2+1:(n+1)*2); 
end
   nr = Mi+1+280;
  Abar = zeros(2,(nr+1)*2);
  Bbar = zeros(2,(nr+1)*2);


  Abar(:,(Mi*2)+1:(Mi+1)*2) = A(:,(2*Mi)+1:(Mi+1)*2);
  
  Bbar(:,(Mi*2)+1:(Mi+1)*2) = B1(:,(2*Mi)+1:(Mi+1)*2); 
  
  for i = (Mi-1):-1:0
     Abar(:,(i*2)+1:(i+1)*2) = A(:,(i*2)+1:(i+1)*2) + (Abar(:,((i+1)*2)+1:(i+2)*2)*G);
     Bbar(:,(i*2)+1:(i+1)*2) = B1(:,(i*2)+1:(i+1)*2) + (Bbar(:,((i+1)*2)+1:(i+2)*2)*G);
  end 
  
  temp = (I - Abar(:,2+1:2*2))^(-1); 
  
  p(1:2) = (P0 * Bbar(:,2+1:2*2))*temp;
  P1 = p
  for i = 2:nr
      temp1 = zeros(1,2); 
      for j = 1:(i-1)
         temp1 = temp1 + (p((j-1)*2+1:j*2) * Abar(:,(i+1-j)*2+1:(i+2-j)*2)); 
      end   
      temp1 = temp1 + (P0 * Bbar(:,i*2+1:(i+1)*2));  
      p((i-1)*2+1:i*2) = temp1 * temp; 
  end 
     
%-------******* Now here solving for queue length distribution   ******** 
    P2 = p(3:4)   
    oc(1) = sum(P0); 
    for i = 1:nr 
       oc(i+1) = sum(p((i-1)*2+1:i*2));
     end 
    probsum = sum(oc)
xax = 0:1:19;
occ = oc(1:20); 
  plot(xax,occ); 
% We are plotting the probability mass function of queue occupancy...