%This program is an event driven simulation of the  the M/G/1 queue, run with and without 
%antithetic variates. The inputs are the message arrival rate, the time duration of the 
%simulation. The output is an estimate of the standard deviation of delay. 
%It is assumed that messages are served in order of arrival.
%For arrival rates, lmbda = 0.89, 0.93, 0.97, 1.01 and 1.05 and for time = 30000
%the simulation can be run for two sets of seeds, one being same, 123, and 123 
%and the other set being 123, and 1234567890
%Example running this program 
%by calling dstd = ex920(1.05,30000,123,123) gives dstd = 1.7931
%and by calling dstd = ex920(1.05,30000,123,1234567890) gives dstd = 1.8038

function [dstd]= ex920(lmda,time,seed1,seed2)
rand('state',seed1);
%initializing the program
currtime=0;%clock value
queue=[0];%vector for messages in buffer
numarr=0;%running variable indicating the number of arrivals
numdep=0;%running variable indicating the number of departures
max =1000;%Upper bound on the number of arriavls in a message transmission.
while ((currtime < time) & (numdep < 100000))
   
%first check if queue is empty
   if queue(1)<1; %empty queue
      currtime=currtime -((1/lmda)*log(rand)); %jumping to time of next arrival
      queue(1)=1;                                            %incrementing the queue
      queue(2)=currtime;                                %arrival time of message to an empty queue     
      numarr=numarr+1;
   end
%At this point, there should be at least one message in the queue.   

%next transmit a message
   if rand < 0.5
     service=0.5;    
   else 
     service=1.0; 
   end 
   currtime=currtime+service;   %time jump to message departure
   newdelay=[currtime-queue(2)] ;
   numdep=numdep+1;
   delay(numdep)=newdelay; %building the delay vector
   queue(1)=queue(1)-1; %accounting for the departure
   queue=[queue(1) queue(3:end)]; %accounting for the departure
   
   %Now we generate the number of arrivals while the message was being
   %transmitted. See equation (9.30) of the text.
   prod=rand;
   thresh=exp(-lmda*service);
  if prod > thresh
   ind=0;   %initializing       
   while thresh < prod  %stepping through the distribution
      ind=ind+1;
      prod=prod*rand;
   end
   %'ind' is the number of Poisson arrivals in the interval.
   
   newarr=sort(service*rand(1,ind))+currtime-service;    %Updating the arrival times. This is a vector 
   %giving the arrival times of message that arrived during a service time.

   queue(1)=queue(1)+ind;    %Updating the queue with number of arrivals.
   
   queue=[queue newarr(1:end)];        %Splicing new arrival times onto the queue 
 end
end 
numdep
%numdep is the number of messages that have departed. A check is numarr=numde+queue(1)
%delay is a vector containing all of the departures. 
d1 = delay;
%d=mean(delay)%mean value of the delay samples
%dvar=var(delay);%variance of the delay

rand('state',seed2);
%initializing the program
currtime=0;%clock value
queue=[0];%vector for messages in buffer
numarr=0;%running variable indicating the number of arrivals
numdep=0;%running variable indicating the number of departures
max =1000;%Upper bound on the number of arriavls in a message transmission.
while ((currtime < time) & (numdep < 100000))
   
%first check if queue is empty
   if queue(1)<1; %empty queue
      currtime=currtime -((1/lmda)*log(1-rand)); %jumping to time of next arrival
      queue(1)=1;                                            %incrementing the queue
      queue(2)=currtime;                                %arrival time of message to an empty queue     
      numarr=numarr+1;
   end
%At this point, there should be at least one message in the queue.   

%next transmit a message
   if (1-rand) < 0.5       % with antithetic variate....
      service=0.5;    
   else 
      service=1.0;  
   end 
   currtime=currtime+service;   %time jump to message departure
   newdelay=[currtime-queue(2)] ;
   numdep=numdep+1;
   delay(numdep)=newdelay; %building the delay vector
   queue(1)=queue(1)-1; %accounting for the departure
   queue=[queue(1) queue(3:end)]; %accounting for the departure
   
   %Now we generate the number of arrivals while the message was being
   %transmitted. See equation (9.30) of the text.
   prod=1-rand;
   thresh=exp(-lmda*service);
  if prod > thresh
   ind=0;   %initializing       
   while thresh < prod  %stepping through the distribution
      ind=ind+1;
      prod=prod*(1-rand);
   end
   %'ind' is the number of Poisson arrivals in the interval.
   
   newarr=sort(service*(1-rand(1,ind)))+currtime-service;    %Updating the arrival times. This is a vector 
   %giving the arrival times of message that arrived during a service time.

   queue(1)=queue(1)+ind;    %Updating the queue with number of arrivals.
   
   queue=[queue newarr(1:end)];        %Splicing new arrival times onto the queue 
 end
end 
numdep
%numdep is the number of messages that have departed. A check is numarr=numde+queue(1)
%delay is a vector containing all of the departures. 
d2 = delay;
if length(d1) < length(d2)
   d2 = d2(1:length(d1));
else 
   d1 = d1(1:length(d2));
end
dstd=std([d1 d2]);%standard deviation of the delay from both sequences...

